package cn.com.qmth.scancentral.controller.scan;

import cn.com.qmth.scancentral.bean.answersave.AnswerPackageDomain;
import cn.com.qmth.scancentral.bean.answersave.AnswerPackageSave;
import cn.com.qmth.scancentral.bean.answersave.AnswerPackageStudent;
import cn.com.qmth.scancentral.entity.BatchEntity;
import cn.com.qmth.scancentral.entity.StudentEntity;
import cn.com.qmth.scancentral.entity.StudentPaperEntity;
import cn.com.qmth.scancentral.enums.LockType;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;

import cn.com.qmth.scancentral.bean.BatchCreateDomain;
import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.bean.answersave.AnswerDomain;
import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.service.BatchService;
import cn.com.qmth.scancentral.vo.BatchFinishVo;
import cn.com.qmth.scancentral.vo.BatchVerifyVo;
import cn.com.qmth.scancentral.vo.SheetUploadVo;
import cn.com.qmth.scancentral.vo.SliceUploadVo;
import cn.com.qmth.scancentral.vo.batch.AnswerSaveVo;
import cn.com.qmth.scancentral.vo.batch.BatchCreateVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.util.List;
import java.util.Map;

@RestController
@Api(tags = "扫描功能-答题卡接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/scan/batch")
@Aac(strict = false, auth = true)
public class ScanBatchController extends BaseController {

    @Autowired
    private BatchService batchService;
    @Autowired
    private ConcurrentService concurrentService;

    @ApiOperation(value = "答题卡扫描批次创建")
    @RequestMapping(value = "/create", method = RequestMethod.POST)
    public BatchCreateVo batchCreate(@Validated @RequestBody BatchCreateDomain domain) {
        return batchService.batchCreate(domain, getAccessUser());
    }

    @ApiOperation(value = "答题卡扫描结果保存")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public AnswerSaveVo batchSave(@Validated @RequestBody AnswerDomain domain) {
        User user = getAccessUser();
        return batchService.batchSave(domain, user);
    }

    @ApiOperation(value = "答题卡整袋扫描结果保存")
    @RequestMapping(value = "/package/save", method = RequestMethod.POST)
    public AnswerPackageSave batchPackageSave(@Validated @RequestBody AnswerPackageDomain domain) {
        User user = getAccessUser();
        concurrentService.getReadWriteLock(LockType.BATCH + "-" + domain.getBatchId()).writeLock().lock();
        BatchEntity batch = batchService.checkBatchStatus(batchService.getById(domain.getBatchId()));
        try{
            Map<String, StudentEntity> studentMap = batchService.validate(batch,domain, user);
            Map<String, List<StudentPaperEntity>> studentPaperMap = batchService.savePaperBatch(batch,user,domain,studentMap);
            batchService.deleteRelation(batch,studentMap);
            batchService.saveAndUpdateStudent(batch,user,studentMap,studentPaperMap);
            return AnswerPackageSave.create(studentMap.size(),batch.getStatus());
        }finally {
            concurrentService.getReadWriteLock(LockType.BATCH + "-" + domain.getBatchId()).writeLock().unlock();
        }
    }

//    @ApiOperation(value = "答题卡扫描历史标记/取消异常")
//    @RequestMapping(value = "/mismatch/toggle", method = RequestMethod.POST)
//    public MismatchToggleVo mismatchToggle(@Validated MismatchToggleDomain domain) {
//        return paperService.mismatchToggle(domain);
//    }

    @ApiOperation(value = "答题卡扫描原图上传")
    @RequestMapping(value = "/sheet/upload", method = RequestMethod.POST)
    public SheetUploadVo sheetUpload(@RequestParam Long batchId, @RequestParam String examNumber,
            @RequestParam Integer paperNumber, @RequestParam Integer pageIndex, @RequestParam MultipartFile file,
            @RequestParam String md5) {
        return batchService.sheetUpload(batchId, examNumber, paperNumber, pageIndex, file, md5);
    }

    @ApiOperation(value = "答题卡扫描裁切图上传")
    @RequestMapping(value = "/slice/upload", method = RequestMethod.POST)
    public SliceUploadVo sliceUpload(@RequestParam Long batchId, @RequestParam String examNumber,
            @RequestParam Integer paperNumber, @RequestParam Integer pageIndex, @RequestParam Integer index,
            @RequestParam MultipartFile file, @RequestParam String md5) {
        return batchService.sliceUpload(batchId, examNumber, paperNumber, pageIndex, index, file, md5);
    }

    @ApiOperation(value = "答题卡扫描批次提交审核/查询审核结果")
    @RequestMapping(value = "/verify", method = RequestMethod.POST)
    public BatchVerifyVo batchVerify(@RequestParam Long id) {
        return batchService.batchVerify(id);
    }

    @ApiOperation(value = "答题卡扫描批次完成")
    @RequestMapping(value = "/finish", method = RequestMethod.POST)
    public BatchFinishVo batchFinish(@RequestParam Long id) {
        return batchService.batchFinish(id,null,null);
    }
}
